using System;

namespace CSharpExamples
{
	/// <summary>
	/// Summary description for StreamsAndXSLT.
	/// </summary>
	public class StreamsAndXSLT
	{
		private System.IO.Stream s = new System.IO.MemoryStream();

		/// <summary>
		/// Example 1
		/// Example showing how to initialize the Stream using a MemoryStream.
		/// only "System.IO.Stream s = new System.IO.MemoryStream();" will be shown
		/// </summary>
		public void InitializeStream()
		{
			System.IO.Stream s = new System.IO.MemoryStream();
		}

		/// <summary>
		/// Example 2
		/// Example showing how to Write bytes to a MemoryStream
		/// </summary>
		public void WriteToStream()
		{
			//Write a letter 'A' to the stream
			s.WriteByte(65);
			//Write a letter 'B' to the stream
			s.WriteByte(66);
			//You can also copy an entire string to a
			//Stream by obtaining the bytes
			byte[] by = System.Text.Encoding.ASCII.GetBytes("This is a String");
			//You must tell it how many bytes you want to Write
			//from the source byte array.
			s.Write(by,0,by.Length);
		}

		/// <summary>
		/// Example 3
		/// Example showing how to Read bytes to a MemoryStream
		/// </summary>
		public void ReadFromStream()
		{
			//set the position back to 0
			s.Position = 0;
			int rb = 0;
			int charCount = 0;
			char c = ' ';
			string st = "";
			//Read the letter 'A' From the stream - 65
			rb = s.ReadByte();
			//Read the letter 'B' From the stream - 66
			rb = s.ReadByte();
			//You may also cast the value to a character - B
			c = (char)rb;
			//You can also read an entire string from the
			//Stream by initializing a byte array and passing it into the
			//Read method. Initilize the byte array the size of the remaining
			//characters
			byte[] by = new byte[s.Length - s.Position];
			//You must tell it how many bytes you want to Read
			//from the Stream
			//charCount will tell you how many bytes were read from the stream
			charCount = s.Read(by,0,by.Length);
			//Use the encoding method to convert a byte array into a String
			st = System.Text.Encoding.ASCII.GetString(by);
		}

		/// <summary>
		/// Example 4
		/// Example showing how to seek for the beginning position.
		/// only "s.Seek(0,System.IO.SeekOrigin.Begin);" will be shown
		/// </summary>
		public void SeekForBeginOfStream()
		{
			System.IO.Stream s = new System.IO.MemoryStream();
			s.Seek(0,System.IO.SeekOrigin.Begin);
		}

		/// <summary>
		/// Example 5
		/// Show how to write String information to the Stream using the StreamWriter
		/// </summary>
		public void WriteDataToStreamWriter()
		{
			//Set the position to zero in the base Stream
			s.Position = 0;
			System.IO.StreamWriter sw = new System.IO.StreamWriter(s);
			sw.WriteLine("This is Line one");
			sw.WriteLine("This is Line two");
			sw.WriteLine("This is Line three");
			sw.Flush();
			//If you were done with the Stream you would need to also Close it
			//sw.Close();
		}

		/// <summary>
		/// Example 6
		/// Show how to Read String information from the Stream
		/// in a loop using the StreamReader
		/// </summary>
		public void ReadDataFromStreamReader()
		{
			//Set the position to zero in the base Stream
			s.Position = 0;
			System.IO.StreamReader sr = new System.IO.StreamReader(s);
			while(sr.Peek() > 0){
				string line = sr.ReadLine();
			}
		}

		/// <summary>
		/// Example 7
		/// Transform XML using two files.
		/// </summary>
		public void TransformUsingFiles()
		{
			//Create the XslTransform.
			System.Xml.Xsl.XslTransform xslt = new System.Xml.Xsl.XslTransform();
  
			//Load the stylesheet.
			xslt.Load("numbers.xsl");

			//Load the XML data file.
			System.Xml.XPath.XPathDocument doc = new System.Xml.XPath.XPathDocument("numbers.xml");

			//Create the XmlTextWriter to output to the console.            
			System.Xml.XmlTextWriter writer = new System.Xml.XmlTextWriter(System.Console.Out);

			//Transform the file.
			xslt.Transform(doc, null, writer);
			//Be careful when using the console, 
			//closing it will not allow you to write any additional information to it
			writer.Close();		
		}


		/// <summary>
		/// Example 8
		/// Transform XML Directly to Stream then use StreamReader
		/// To retrieve the information back.
		/// </summary>
		/// <returns>String of the xml Result</returns>
		public string TransformUsingStreamAsOutputReturningString()
		{
			//Create the XslTransform.
			System.Xml.Xsl.XslTransform xslt = new System.Xml.Xsl.XslTransform();
  
			//Load the stylesheet.
			xslt.Load("numbers.xsl");

			//Load the XML data file.
			System.Xml.XPath.XPathDocument doc = new System.Xml.XPath.XPathDocument("numbers.xml");

			//Create the Stream to place the output.            
			System.IO.Stream str = new System.IO.MemoryStream();

			//Transform the file.
			xslt.Transform(doc, null, str);
			
			//flush and set the position to 0
			str.Flush();
			str.Position = 0;
			
			//Create a StreamReader to Read the Stream and Return the String
			System.IO.StreamReader sr = new System.IO.StreamReader(str);
			string xmlOut = sr.ReadToEnd();
			//Close the StreamReader and the base Stream
			sr.Close();
			return xmlOut;
		}

		/// <summary>
		/// Example 9
		/// Transform an XML Document already in Memory
		/// Using the XMLTextWriter
		/// Then use XMLTextReader to Read the Information Back
		/// </summary>
		/// <returns></returns>
		public string TransformXMLDocUsingXmLTextObjectsAndReturnString()
		{
			//Create the XslTransform.
			System.Xml.Xsl.XslTransform xslt = new System.Xml.Xsl.XslTransform();
  
			//Load the stylesheet that creates XML Output.
			xslt.Load("numbersXML.xsl");

			//Load the XML data file.
			System.Xml.XmlDocument doc = new System.Xml.XmlDocument();
			doc.Load("numbers.xml");

			//Create the Stream to place the output.            
			System.IO.Stream str = new System.IO.MemoryStream();
			System.Xml.XmlWriter xw = new System.Xml.XmlTextWriter(str,System.Text.Encoding.UTF8);

			//Transform the file.
			xslt.Transform(doc, null, xw);
			
			//flush and set the position to 0
			xw.Flush();
			
			str.Position = 0;
			
			//Create a XMLTextReader to Read the XML Back and Return the String of nodes
			System.Xml.XmlReader xr = new System.Xml.XmlTextReader(str);
			//Use the StringBuilder to append the node information
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			//Just loop through the nodes, both the open and close nodes are returned
			while (xr.Read()){
				sb.Append ("Type: " + xr.NodeType.ToString() + " Name: " + xr.Name + System.Environment.NewLine);
			}
			//Close the XMLTextReader and the base Stream
			xr.Close();
			return sb.ToString();
		}




		public StreamsAndXSLT()
		{
			//
			// TODO: Add constructor logic here
			//
		}
	}
}
