Public Class StreamsAndXSLT

    '/// <summary>
    '/// Example 1
    '/// Example showing how to initialize the Stream using a MemoryStream.
    '/// only "Dim s As System.IO.Stream = New System.IO.MemoryStream()" will be shown
    '/// </summary>
    Private s As System.IO.Stream = New System.IO.MemoryStream()

    Public Sub InitializeStream()
        Dim s As System.IO.Stream = New System.IO.MemoryStream()
    End Sub

    '/// <summary>
    '/// Example 2
    '/// Example showing how to write bytes to a MemoryStream
    '/// </summary>
    Public Sub WriteToStream()
        'Write a letter 'A' to the stream
        s.WriteByte(65)
        'Write a letter 'B' to the stream
        s.WriteByte(66)
        'You can also copy an entire string to a
        'Stream by obtaining the bytes
        Dim by() As Byte = System.Text.Encoding.ASCII.GetBytes("This is a String")
        'You must tell it how many bytes you want to write
        'from the source byte array.
        s.Write(by, 0, by.Length)
    End Sub

    '/// <summary>
    '/// Example 3
    '/// Example showing how to Read bytes to a MemoryStream
    '/// </summary>
    Public Sub ReadFromStream()
        'set the position back to 0
        s.Position = 0
        Dim rb As Integer
        Dim charCount As Integer
        Dim c As Char = " "c
        Dim st As String = ""
        'Read the letter 'A' From the stream - 65
        rb = s.ReadByte()
        'Read the letter 'B' From the stream - 66
        rb = s.ReadByte()
        'You may also cast the value to a character - B
        c = System.Convert.ToChar(rb)
        'You can also read an entire string from the
        'Stream by initializing a byte array and passing it into the
        'Read method. Initilize the byte array the size of the remaining
        'characters
        Dim length As Integer = System.Convert.ToInt32(s.Length - s.Position)
        Dim by(length - 1) As Byte 'Arrays in VB are upper index not number of items
        'You must tell it how many bytes you want to Read
        'from the Stream
        'charCount will tell you how many bytes were read from the stream
        charCount = s.Read(by, 0, by.Length)
        'Use the encoding method to convert a byte array into a String
        st = System.Text.Encoding.ASCII.GetString(by)
    End Sub

    '/// <summary>
    '/// Example 4
    '/// Example showing how to seek for the beginning position.
    '/// only "s.Seek(0,System.IO.SeekOrigin.Begin);" will be shown
    '/// </summary>
    Public Sub SeekForBeginOfStream()
        Dim s As System.IO.Stream = New System.IO.MemoryStream()
        s.Seek(0, System.IO.SeekOrigin.Begin)
    End Sub

    '/// <summary>
    '/// Example 5
    '/// Show how to write String information to the Stream
    '/// </summary>
    Public Sub WriteDataToStreamWriter()
        'Set the position to zero in the base Stream
        s.Position = 0
        Dim sw As System.IO.StreamWriter = New System.IO.StreamWriter(s)
        sw.WriteLine("This is Line one")
        sw.WriteLine("This is Line two")
        sw.WriteLine("This is Line three")
        sw.Flush()
        'If you were done with the Stream you would need to also Close it
        'sw.Close()
    End Sub

    '/// <summary>
    '/// Example 6
    '/// Show how to Read String information from the Stream
    '/// in a loop using the StreamReader
    '/// </summary>
    Public Sub ReadDataFromStreamReader()
        'Set the position to zero in the base Stream
        s.Position = 0
        Dim sr As System.IO.StreamReader = New System.IO.StreamReader(s)
        Do While (sr.Peek() > 0)
            Dim line As String = sr.ReadLine()
        Loop
    End Sub


    '/// <summary>
    '/// Example 7
    '/// Transform XML using two files.
    '/// </summary>
    Public Sub TransformUsingFiles()
        'Create the XslTransform.
        Dim xslt As System.Xml.Xsl.XslTransform = New System.Xml.Xsl.XslTransform()

        'Load the stylesheet.
        xslt.Load("numbers.xsl")

        'Load the XML data file.
        Dim doc As System.Xml.XPath.XPathDocument = New System.Xml.XPath.XPathDocument("numbers.xml")

        'Create the XmlTextWriter to output to the console.             
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(System.Console.Out)

        'Transform the file.
        xslt.Transform(doc, Nothing, writer)
        'Be careful when using the console, 
        'closing it will not allow you to write any additional information to it
        writer.Close()
    End Sub


    '/// <summary>
    '/// Example 8
    '/// Transform XML Directly to Stream then use StreamReader
    '/// To retrieve the information back.
    '/// </summary>
    '/// <returns>String of the xml Result</returns>
    Public Function TransformUsingStreamAsOutputReturningString() As String
        'Create the XslTransform.
        Dim xslt As System.Xml.Xsl.XslTransform = New System.Xml.Xsl.XslTransform()

        'Load the stylesheet.
        xslt.Load("numbers.xsl")

        'Load the XML data file.
        Dim doc As System.Xml.XPath.XPathDocument = New System.Xml.XPath.XPathDocument("numbers.xml")

        'Create the Stream to place the output.             
        Dim str As System.IO.Stream = New System.IO.MemoryStream()

        'Transform the file.
        xslt.Transform(doc, Nothing, str)
        'flush and set the position to 0
        str.Flush()
        str.Position = 0

        'Create a StreamReader to Read the Stream and Return the String
        Dim sr As System.IO.StreamReader = New System.IO.StreamReader(str)
        Dim xmlOut As String = sr.ReadToEnd()

        'Close the StreamReader and the base Stream
        sr.Close()
        Return xmlOut
    End Function

    '/// <summary>
    '/// Example 9
    '/// Transform an XML Document already in Memory
    '/// Using the XMLTextWriter
    '/// Then use XMLTextReader to Read the Information Back
    '/// </summary>
    '/// <returns></returns>
    Public Function TransformXMLDocUsingXmLTextObjectsAndReturnString() As String
        'Create the XslTransform.
        Dim xslt As System.Xml.Xsl.XslTransform = New System.Xml.Xsl.XslTransform()

        'Load the stylesheet.
        xslt.Load("numbersXML.xsl")

        'Load the XML data file.
        Dim doc As System.Xml.XmlDocument = New System.Xml.XmlDocument()
        doc.Load("numbers.xml")

        'Create the Stream to place the output.         
        Dim str As System.IO.Stream = New System.IO.MemoryStream()
        Dim xw As System.Xml.XmlWriter = New System.Xml.XmlTextWriter(str, System.Text.Encoding.UTF8)


        'Transform the file.
        xslt.Transform(doc, Nothing, xw)

        'flush and set the position to 0
        xw.Flush()

        str.Position = 0

        'Create a XMLTextReader to Read the XML Back and Return the String of nodes
        Dim xr As System.Xml.XmlReader = New System.Xml.XmlTextReader(str)
        'Use the StringBuilder to append the node information
        Dim sb As System.Text.StringBuilder = New System.Text.StringBuilder()
        'Just loop through the nodes, both the open and close nodes are returned
        Do While (xr.Read())
            sb.Append("Type: " & xr.NodeType.ToString() & " Name: " & xr.Name & System.Environment.NewLine)
        Loop
        'Close the XMLTextReader and the base Stream
        xr.Close()
        Return sb.ToString()
    End Function


End Class
